/*
    Rekollect: A note taking application
    Copyright (C) 2009  Jason Jackson <jaxhoo@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#include "notebrowserwindow.h"
#include "notecollection.h"
#include "windowcollection.h"
#include "systemtrayicon.h"

#include <QtGui/QTableView>
#include <QtGui/QHeaderView>
#include <QtGui/QCloseEvent>
#include <QtGui/QSortFilterProxyModel>

#include <KIcon>
#include <KApplication>
#include <KToolBar>
#include <KActionCollection>
#include <KAction>
#include <KStandardAction>
#include <KLocalizedString>

NoteBrowserWindow::NoteBrowserWindow(QWidget *parent) : KXmlGuiWindow(parent)
{
    setAttribute(Qt::WA_DeleteOnClose, false);
    setWindowIcon(KIcon("rekollect"));
    setupWindow();
    setupActions();
}

void NoteBrowserWindow::closeEvent(QCloseEvent *event)
{
    // Do not destroy window on close
    hide();
    event->ignore();
}

void NoteBrowserWindow::setupWindow()
{
    m_noteCollection = new NoteCollection(this);
    m_sortableProxyModel = new QSortFilterProxyModel(this);
    m_sortableProxyModel->setSourceModel(m_noteCollection);
    m_sortableProxyModel->setDynamicSortFilter(true);
    m_sortableProxyModel->setSortRole(Qt::EditRole);
    m_noteTableView = new QTableView;
    m_noteTableView->setModel(m_sortableProxyModel);
    m_noteTableView->resizeColumnsToContents();
    m_noteTableView->horizontalHeader()->stretchLastSection();
    m_noteTableView->setEditTriggers(QTableView::NoEditTriggers);
    m_noteTableView->setSelectionMode(QTableView::SingleSelection);
    m_noteTableView->setSelectionBehavior(QTableView::SelectRows);
    m_noteTableView->setSortingEnabled(true);
    m_noteTableView->sortByColumn(NoteCollection::ModificationTimeColumn, Qt::DescendingOrder);
    connect(m_noteTableView, SIGNAL(activated(QModelIndex)), SLOT(openNote(QModelIndex)));
    setCentralWidget(m_noteTableView);

    m_windowCollection = new WindowCollection(this);
    m_windowCollection->setNoteCollection(m_noteCollection);
    m_noteCollection->setWindowCollection(m_windowCollection);
    setupTray();
}

void NoteBrowserWindow::setupActions()
{
    m_newNoteAction = new KAction(i18nc("Menu entry name.", "Create Note"), this);
    m_newNoteAction->setIcon(KIcon("document-new"));
    connect(m_newNoteAction, SIGNAL(triggered()), m_windowCollection, SLOT(createNewNoteWindow()));
    actionCollection()->addAction("new", m_newNoteAction);
    KStandardAction::quit(kapp, SLOT(quit()), actionCollection());
    toolBar()->setVisible(false);
    setupGUI();
}

void NoteBrowserWindow::setupTray()
{
    m_systemTrayIcon = new SystemTrayIcon(this);
    m_systemTrayIcon->show();
    for (int i = 0; i < m_noteCollection->rowCount(); ++i)
    {
        QString fileName = m_noteCollection->item(i, NoteCollection::FileNameColumn)->text();
        QString documentName = m_noteCollection->item(i, NoteCollection::DocumentNameColumn)->text();
        m_systemTrayIcon->addDocumentAction(fileName, documentName);
    }
    connect(m_noteCollection, SIGNAL(noteCreated(QString,QString)), m_systemTrayIcon, SLOT(addDocumentAction(QString,QString)));
    connect(m_noteCollection, SIGNAL(noteRemoved(QString)), m_systemTrayIcon, SLOT(removeDocumentAction(QString)));
    connect(m_noteCollection, SIGNAL(noteNameChanged(QString,QString)), m_systemTrayIcon, SLOT(renameDocumentAction(QString,QString)));
    connect(m_systemTrayIcon, SIGNAL(openNoteRequested(QString)), m_windowCollection, SLOT(openNoteWindow(QString)));
    connect(m_systemTrayIcon, SIGNAL(createNoteRequested()), m_windowCollection, SLOT(createNewNoteWindow()));
}

void NoteBrowserWindow::openNote(const QModelIndex &index)
{
    QModelIndex realIndex = m_sortableProxyModel->mapToSource(index);
    m_windowCollection->openNoteWindow(m_noteCollection->fileNameAt(realIndex));
}
