/*
    Rekollect: A note taking application
    Copyright (C) 2009, 2010  Jason Jackson <jaxhoo@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#include "windowcollection.h"

#include "note.h"
#include "notewindow.h"
#include "notecollection.h"
#include "linkeditdialog.h"
#include "noteeditor.h"

#include <KRun>
#include <KMessageBox>
#include <KLocalizedString>
#include <KStandardDirs>

WindowCollection::WindowCollection(QObject *parent) : QObject(parent) {}

void WindowCollection::setNoteCollection(NoteCollection* noteCollection)
{
    m_noteCollection = noteCollection;
}

void WindowCollection::openNoteWindow(const QString &fileName)
{
    if (m_windows.contains(fileName)) {
        NoteWindow *window = m_windows.value(fileName);
        if (window->isMinimized()) {
            window->showNormal();
        }
        window->raise();
        window->activateWindow();
        return;
    }

    Note *note = m_noteCollection->openNote(fileName);
    if (note) {
        openNoteWindow(m_noteCollection->openNote(fileName));
    }
}

void WindowCollection::openNoteWindow(Note *note)
{
    NoteWindow *noteWindow = new NoteWindow(note, this);
    connect(noteWindow, SIGNAL(closeRequested(QString)), SLOT(removeNoteWindow(QString)));
    connect(noteWindow, SIGNAL(newNoteRequested()), SLOT(createNewNoteWindow()));
    connect(noteWindow, SIGNAL(linkTriggered(QString,QString)), SLOT(linkTriggered(QString,QString)));
    connect(noteWindow->editor(), SIGNAL(noteChanged(Note*)), m_noteCollection, SLOT(saveNote(Note*)));
    m_windows.insert(note->fileName(), noteWindow);
    noteWindow->show();
}

QString WindowCollection::createNewNoteWindow(const QString &newNoteName)
{
    // Request a Note from the NoteCollection
    Note *note = m_noteCollection->createNewNote(newNoteName);
    // OpenNoteWindow with newly created note
    openNoteWindow(note);
    return note->fileName();
}

void WindowCollection::removeNoteWindow(const QString &fileName)
{
    if (m_windows.contains(fileName)) {
        m_windows.remove(fileName);
    }
}

void WindowCollection::linkTriggered(const QString &linkUrl, const QString &linkName)
{
    if (linkUrl.startsWith("rekollect:")) {
        QString fileName = linkUrl.section(':', 1);
        QString fullFilePath = KStandardDirs::locateLocal("app_notes", fileName);
        if (m_noteCollection->noteExists(fullFilePath)) {
            openNoteWindow(fullFilePath);
        } else {
            NoteWindow *window = qobject_cast<NoteWindow *>(sender());
            int result = KMessageBox::questionYesNoCancel(
                window, i18n("<b>%1</b> does not exist. Do you "
                                "want to remove or edit the link?")
                                .arg(linkName),
                i18n("Question"),
                KGuiItem(i18n("Remove Link")),
                KGuiItem(i18n("Edit Link"))
            );
            if (result == KMessageBox::Yes) {
                // Remove Link button is selected
                window->removeCurrentLink();
            } else if (result == KMessageBox::No) {
                // Create New Note is selected
                window->editLink();
            }
        }
    } else {
        KRun *run = new KRun(linkUrl, qobject_cast<QWidget *>(parent()));
        Q_UNUSED(run)
    }
}
